using System;
using System.Collections;
using TXTextControl;
using Keyoti.RapidSpell;
using System.Windows.Forms;

namespace TX_Text_Control_Words
{
    /// <summary>
    /// TXHelper - glues TX and RapidSpell together.
    /// </summary>
    public class TXHelper
    {
        RapidSpellAsYouType ayt;
        RapidSpellDialog dia;
        TXTextControl.TextControl txc;
        TXTextControlAdapter dialogAdapter;
        TXTextControlAdapter[] adapters;
        bool isTextFrameActive = false;
        ContextMenuStrip defaultMenu;

        public bool IsTextFrameActive { get { return isTextFrameActive; } }


        /// <summary>
        /// Configures the spell checkers to work with the text control, pass a null
        /// spell checker object if not being used.
        /// </summary>
        public TXHelper(RapidSpellAsYouType ayt, RapidSpellDialog dia, TXTextControl.TextControl txc, ContextMenuStrip defaultMenu)
        {
            this.ayt = ayt;
            this.dia = dia;
            this.txc = txc;
            this.defaultMenu = defaultMenu;
           
            if (dia != null)
            {
                dialogAdapter = new TXTextControlAdapter(txc);
                dialogAdapter.WorkWithCurrentlyActiveElementOnly = true;
                dia.ThirdPartyTextComponentToCheck = dialogAdapter;
                dia.SpellCheckStarted += new Keyoti.RapidSpell.RapidSpellDialog.SpellCheckEventHandler(dia_SpellCheckStarted);
                dia.SpellCheckFinished += new Keyoti.RapidSpell.RapidSpellDialog.SpellCheckEventHandler(dia_SpellCheckFinished);
            }

            if (ayt != null)
            {
                this.txc.TextFrameCreated += new TextFrameEventHandler(txc_TextFrameCreated);
                adapters = (TXTextControlAdapter[])getBasisAdapters().ToArray(typeof(TXTextControlAdapter));
                this.ayt.UpdateAllTextBoxes = false;
                this.ayt.TextComponents = adapters;
            }

            this.txc.TextFrameActivated += new TextFrameEventHandler(txc_TextFrameActivated);
            this.txc.TextFrameDeactivated += new TextFrameEventHandler(txc_TextFrameDeactivated);
        }



        public void Dispose()
        {

            if (dia != null)
            {
                dia.SpellCheckStarted -= new Keyoti.RapidSpell.RapidSpellDialog.SpellCheckEventHandler(dia_SpellCheckStarted);
                dia.SpellCheckFinished -= new Keyoti.RapidSpell.RapidSpellDialog.SpellCheckEventHandler(dia_SpellCheckFinished);
            }

            if (txc != null && !txc.IsDisposed)
            {
                this.txc.TextFrameCreated -= new TextFrameEventHandler(txc_TextFrameCreated);
                this.txc.TextFrameActivated -= new TextFrameEventHandler(txc_TextFrameActivated);
                this.txc.TextFrameDeactivated -= new TextFrameEventHandler(txc_TextFrameDeactivated);
            }
        }

        ArrayList getBasisAdapters()
        {
            ArrayList adaps = new ArrayList();
            TXTextControlAdapter adap;

            //add main adapter
            adap = new TXTextControlAdapter(this.txc);
            adaps.Add(adap);
            adap.ContextMenuStripDefault = defaultMenu;

            //add adapters for headers/footers
            if (txc.HeadersAndFooters != null)
            {
                foreach (HeaderFooter hf in this.txc.HeadersAndFooters)
                {
                    adap = new TXTextControlAdapter(this.txc);
                    adap.HeaderFooter = hf;
                    adap.ContextMenuStripDefault = defaultMenu;
                    adaps.Add(adap);
                }
            }
            return adaps;
        }

        private void dia_SpellCheckStarted(object sender, Keyoti.RapidSpell.Event.SpellCheckEventArgs e)
        {
            dialogAdapter.IgnoreHeaderFooterDeactivationEvents = true;
        }

        private void dia_SpellCheckFinished(object sender, Keyoti.RapidSpell.Event.SpellCheckEventArgs e)
        {
            dialogAdapter.IgnoreHeaderFooterDeactivationEvents = false;
        }

        //used for AYT only, this is only called when text frames come from the CLIPBOARD
        private void txc_TextFrameCreated(object sender, TextFrameEventArgs e)
        {
            OnTextFrameCreated(e.TextFrame);
        }

        //used for AYT only
        /// <summary>
        /// Call when adding a TextFrame
        /// </summary>
        /// <param name="tf"></param>
        public void OnTextFrameCreated(TextFrame tf)
        {
            TXTextControlAdapter adap = new TXTextControlAdapter(this.txc);
            adap.TextFrame = tf;
            adap.ContextMenuStripDefault = defaultMenu;

            AddAdapter(adap);

        }


        void AddAdapter(TXTextControlAdapter newAdapter)
        {
            ArrayList basis = new ArrayList();

            TXTextControlAdapter adap;
            //refresh old adapters, note we must create new adapters as old ones are retired below
            foreach (TXTextControlAdapter a in adapters)
            {
                adap = new TXTextControlAdapter(this.txc);
                adap.SetWith(a);
                adap.ContextMenuStripDefault = defaultMenu;
                basis.Add(adap);
            }


            //add new adapter
            basis.Add(newAdapter);


            adapters = (TXTextControlAdapter[])basis.ToArray(typeof(TXTextControlAdapter));

            //retire the old ones
            ayt.TextComponents = null;
            //add the new ones
            ayt.TextComponents = adapters;
        }


        private void txc_TextFrameActivated(object sender, TextFrameEventArgs e)
        {
            isTextFrameActive = true;
        }

        private void txc_TextFrameDeactivated(object sender, TextFrameEventArgs e)
        {
            isTextFrameActive = false;
        }

        public void OnDocumentOpen()
        {
            ButtonBar bb = txc.ButtonBar;
            RulerBar rb = txc.RulerBar;
            RulerBar vrb = txc.VerticalRulerBar;
            TXTextControl.StatusBar sb = txc.StatusBar;

            txc.ButtonBar = null;
            txc.RulerBar = null;
            txc.VerticalRulerBar = null;
            txc.StatusBar = null;

            adapters[0].OnTextChanged(txc, EventArgs.Empty);

            txc.ButtonBar = bb;
            txc.RulerBar = rb;
            txc.VerticalRulerBar = vrb;
            txc.StatusBar = sb;
        }
    }
}
