using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Reflection;
using System.Windows.Forms;

namespace TX_Text_Control_Words
{
    public partial class frmMain : Form
    {
        #region "  Form Properties  "

        private FileHandler FileHandler1;
        private bool m_LoadFileOnCreate = false;
        private TXTextControl.HeaderFooter m_ActiveHeaderFooter;
        //RapidSpell-
        TXHelper helper;
        //-

        public frmMain(string[] Args)
        {
            InitializeComponent();
            textControl1.ButtonBar = buttonBar1;
            textControl1.RulerBar = rulerBar1;
            textControl1.VerticalRulerBar = rulerBar2;
            textControl1.StatusBar = statusBar1;

            FileHandler1 = new FileHandler(this);

            

            if (Args.Length != 0)
            {
                FileHandler1.DocumentFileName = Args[0];
                m_LoadFileOnCreate = true;
            }
        }

        public bool CanCopy
        {
            get { return textControl1.CanCopy; }
        }

        public bool CanPaste
        {
            get { return textControl1.CanPaste; }
        }

        public bool CanUndo
        {
            get { return textControl1.CanUndo; }
        }

        public bool CanRedo
        {
            get { return textControl1.CanRedo; }
        }

        public bool LoadFileOnCreate
        {
            set { m_LoadFileOnCreate = value; }
        }

        #endregion

        #region " Drag and Drop "

        private void frmMain_DragOver(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false) == true)
                e.Effect = DragDropEffects.All;
        }

        private void frmMain_DragDrop(object sender, DragEventArgs e)
        {
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);

            System.Windows.Forms.DialogResult Result;

            if (FileHandler1.DocumentDirty)
            {
                Result = MessageBox.Show("Save changes to " + this.Text + "?", "Question", MessageBoxButtons.YesNoCancel);
                if (Result == DialogResult.Yes)
                {
                    FileHandler1.FileSave();
                }
                else if (Result == DialogResult.Cancel)
                    return;
            }

            foreach (string file in files)
            {
                FileHandler1.DocumentFileName = file;
                this.Text = FileHandler1.DocumentFileName;
                FileHandler1.FileOpen();
                // loop through all files, if MDI is implemented
                break;
            }
        }

        #endregion

        #region " File menu event handlers "

        private void mnuHelp_AboutTXTextControlWords_Click(object sender, EventArgs e)
        {
            About();
        }

        private void mnuFile_Open_Click(object sender, EventArgs e)
        {
            FileOpen();
        }

        private void mnuFile_DropDownOpening(object sender, EventArgs e)
        {
            FileHandler1.InitRecentFiles();
        }

        private void mnuFile_New_Click(object sender, EventArgs e)
        {
            FileNew();
        }

        private void mnuFile_Save_Click(object sender, System.EventArgs e)
        {
            FileSave();
        }

        private void mnuFile_SaveAs_Click(object sender, System.EventArgs e)
        {
            try
            {
                FileHandler1.FileSaveAs();
                if (FileHandler1.DocumentFileName != "")
                    this.Text = FileHandler1.DocumentFileName;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "TX Text Control Words");
            }
        }

        private void mnuFile_Export_Click(object sender, System.EventArgs e)
        {
            try
            {
                FileHandler1.Export();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "TX Text Control Words");
            }
        }

        private void mnuFile_PageSetup_Click(object sender, System.EventArgs e)
        {
            textControl1.SectionFormatDialog(0);
        }

        private void mnuFile_PrintPreview_Click(object sender, System.EventArgs e)
        {
            PrintPreview();
        }

        private void mnuFile_Print_Click(object sender, System.EventArgs e)
        {
            Print();
        }

        private void mnuFile_Exit_Click(object sender, EventArgs e)
        {
            Close();
        }

        #endregion

        #region " Table menu event handlers "

        private void mnuTable_Popup(object sender, System.EventArgs e)
        {
            TXTextControl.Table ThisTable = textControl1.Tables.GetItem();

            mnuTable_Properties.Enabled = (ThisTable != null);
            mnuTable_GridLines.Checked = textControl1.Tables.GridLines;

            if (ThisTable != null)
                mnuTable_Split.Enabled = ThisTable.CanSplit;
            else
                mnuTable_Split.Enabled = false;
        }

        private void mnuTable_Insert_Popup(object sender, System.EventArgs e)
        {
            mnuTable_Insert_Table.Enabled = textControl1.Tables.CanAdd;

            TXTextControl.Table TableAtInputPosition = textControl1.Tables.GetItem();
            if (TableAtInputPosition == null)
            {
                mnuTable_Insert_ColumnToTheLeft.Enabled = false;
                mnuTable_Insert_ColumnToTheRight.Enabled = false;
                mnuTable_Insert_RowAbove.Enabled = false;
                mnuTable_Insert_RowBelow.Enabled = false;
            }
            else
            {
                mnuTable_Insert_ColumnToTheLeft.Enabled = TableAtInputPosition.Columns.CanAdd;
                mnuTable_Insert_ColumnToTheRight.Enabled = TableAtInputPosition.Columns.CanAdd;
                mnuTable_Insert_RowAbove.Enabled = TableAtInputPosition.Rows.CanAdd;
                mnuTable_Insert_RowBelow.Enabled = TableAtInputPosition.Rows.CanAdd;
            }
        }

        private void mnuTable_Insert_Table_Click(object sender, System.EventArgs e)
        {
            frmInsertTable InsertTableDialog = new frmInsertTable();

            InsertTableDialog.tx = textControl1;
            InsertTableDialog.ShowDialog();
        }

        private void mnuTable_Insert_ColumnToTheLeft_Click(object sender, System.EventArgs e)
        {
            textControl1.Tables.GetItem().Columns.Add(TXTextControl.TableAddPosition.Before);
        }

        private void mnuTable_Insert_ColumnToTheRight_Click(object sender, System.EventArgs e)
        {
            textControl1.Tables.GetItem().Columns.Add(TXTextControl.TableAddPosition.After);
        }

        private void mnuTable_Insert_RowAbove_Click(object sender, System.EventArgs e)
        {
            textControl1.Tables.GetItem().Rows.Add(TXTextControl.TableAddPosition.Before, 1);
        }

        private void mnuTable_Insert_RowBelow_Click(object sender, System.EventArgs e)
        {
            textControl1.Tables.GetItem().Rows.Add(TXTextControl.TableAddPosition.After, 1);
        }

        private void mnuTable_Delete_Popup(object sender, System.EventArgs e)
        {
            TXTextControl.Table TableAtInputPosition = textControl1.Tables.GetItem();

            if (TableAtInputPosition == null)
            {
                mnuTable_Delete_Table.Enabled = false;
                mnuTable_Delete_Column.Enabled = false;
                mnuTable_Delete_Rows.Enabled = false;
            }
            else
            {
                mnuTable_Delete_Table.Enabled = TableAtInputPosition.Columns.CanRemove;
                mnuTable_Delete_Column.Enabled = TableAtInputPosition.Columns.CanRemove;
                mnuTable_Delete_Rows.Enabled = TableAtInputPosition.Rows.CanRemove;
            }
        }

        private void mnuTable_Delete_Table_Click(object sender, System.EventArgs e)
        {
            textControl1.Tables.Remove();
        }

        private void mnuTable_Delete_Column_Click(object sender, System.EventArgs e)
        {
            textControl1.Tables.GetItem().Columns.Remove();
        }

        private void mnuTable_Delete_Rows_Click(object sender, System.EventArgs e)
        {
            textControl1.Tables.GetItem().Rows.Remove();
        }

        private void mnuTable_Split_Above_Click(object sender, System.EventArgs e)
        {
            textControl1.Tables.GetItem().Split(TXTextControl.TableAddPosition.Before);
        }

        private void mnuTable_Split_Below_Click(object sender, System.EventArgs e)
        {
            textControl1.Tables.GetItem().Split(TXTextControl.TableAddPosition.After);
        }

        private void mnuTable_Select_Popup(object sender, System.EventArgs e)
        {
            TXTextControl.Table TableAtInputPosition = null;
            TXTextControl.TableRow RowAtInputPosition = null;
            TXTextControl.TableCell CellAtInputPosition = null;

            TableAtInputPosition = textControl1.Tables.GetItem();
            if (TableAtInputPosition != null)
            {
                RowAtInputPosition = TableAtInputPosition.Rows.GetItem();
                CellAtInputPosition = TableAtInputPosition.Cells.GetItem();
            }

            mnuTable_Select_Table.Enabled = (TableAtInputPosition != null);
            mnuTable_Select_Row.Enabled = (RowAtInputPosition != null);
            mnuTable_Select_Cell.Enabled = (CellAtInputPosition != null);
        }

        private void mnuTable_Select_Table_Click(object sender, System.EventArgs e)
        {
            textControl1.Tables.GetItem().Select();
        }

        private void mnuTable_Select_Row_Click(object sender, System.EventArgs e)
        {
            textControl1.Tables.GetItem().Rows.GetItem().Select();
        }

        private void mnuTable_Select_Cell_Click(object sender, System.EventArgs e)
        {
            textControl1.Tables.GetItem().Cells.GetItem().Select();
        }

        private void mnuTable_GridLines_Click(object sender, System.EventArgs e)
        {
            textControl1.Tables.GridLines = !textControl1.Tables.GridLines;
        }

        private void mnuTable_Properties_Click(object sender, System.EventArgs e)
        {
            textControl1.TableFormatDialog();
        }

        #endregion

        #region " Options menu event handlers "

        private void mnuOptions_HTML_Click(object sender, System.EventArgs e)
        {
            frmHTMLOptions HTMLOptionsDialog = new frmHTMLOptions();

            HTMLOptionsDialog.FileHandler1 = FileHandler1;
            HTMLOptionsDialog.ShowDialog();
        }

        private void mnuOptions_PDF_Click(object sender, System.EventArgs e)
        {
            frmPDFOptions PDFOptionsDialog = new frmPDFOptions();

            PDFOptionsDialog.FileHandler1 = FileHandler1;
            PDFOptionsDialog.ShowDialog();
        }

        #endregion

        #region " Format menu event handlers "

        private void mnuFormat_Popup(object sender, System.EventArgs e)
        {
            mnuFormat_Image.Enabled = (textControl1.Images.GetItem() != null);
            mnuFormat_TextFrame.Enabled = (textControl1.TextFrames.GetItem() != null);
            mnuFormat_TextFrameMarkerLines.Checked = textControl1.TextFrameMarkerLines;
            mnuFormat_DocumentTargetMarkers.Checked = textControl1.DocumentTargetMarkers;
        }

        private void mnuFormat_Character_Click(object sender, System.EventArgs e)
        {
            textControl1.FontDialog();
        }

        private void mnuFormat_Paragraph_Click(object sender, System.EventArgs e)
        {
            textControl1.ParagraphFormatDialog();
        }

        private void mnuFormat_Tabs_Click(object sender, System.EventArgs e)
        {
            textControl1.TabDialog();
        }

        private void mnuFormat_Styles_Click(object sender, System.EventArgs e)
        {
            textControl1.FormattingStylesDialog();
        }

        private void mnuFormat_HeadersFooters_Click(object sender, EventArgs e)
        {
            textControl1.SectionFormatDialog(1);
        }

        private void mnuFormat_Image_Click(object sender, System.EventArgs e)
        {
            textControl1.ImageAttributesDialog();
        }

        private void mnuFormat_TextFrame_Click(object sender, System.EventArgs e)
        {
            textControl1.TextFrameAttributesDialog();
        }

        private void mnuFormat_TextFrameMarkerLines_Click(object sender, System.EventArgs e)
        {
            textControl1.TextFrameMarkerLines = !textControl1.TextFrameMarkerLines;
        }

        private void mnuFormat_DocumentTargetMarkers_Click(object sender, EventArgs e)
        {
            textControl1.DocumentTargetMarkers = !textControl1.DocumentTargetMarkers;
        }

        private void mnuFormat_TextColor_Click(object sender, System.EventArgs e)
        {
            if (ColorDialog1.ShowDialog() == DialogResult.OK)
            {
                textControl1.Selection.ForeColor = ColorDialog1.Color;
            }
        }

        private void mnuFormat_TextbackgroundColor_Click(object sender, System.EventArgs e)
        {
            if (ColorDialog1.ShowDialog() == DialogResult.OK)
            {
                textControl1.Selection.TextBackColor = ColorDialog1.Color;
            }
        }

        private void mnuFormat_DocumentBackgroundColor_Click(object sender, System.EventArgs e)
        {
            if (ColorDialog1.ShowDialog() == DialogResult.OK)
            {
                textControl1.BackColor = ColorDialog1.Color;
            }
        }

        private void mnuFormat_BulletsAndNumbering_DropDownOpening(object sender, EventArgs e)
        {
            mnuFormat_BulletsAndNumbering_IncreaseLevel.Enabled = (!(textControl1.Selection.ListFormat.Type == TXTextControl.ListType.None)) && (textControl1.Selection.ListFormat.Level < TXTextControl.ListFormat.MaxLevel);
            mnuFormat_BulletsAndNumbering_DecreaseLevel.Enabled = (!(textControl1.Selection.ListFormat.Type == TXTextControl.ListType.None)) && (textControl1.Selection.ListFormat.Level > 1);
        }

        private void mnuFormat_BulletsAndNumbering_Attributes_Click(object sender, System.EventArgs e)
        {
            textControl1.ListFormatDialog();
        }

        private void mnuFormat_BulletsAndNumbering_IncreaseLevel_Click(object sender, System.EventArgs e)
        {
                textControl1.Selection.ListFormat.Level += 1;
                textControl1.Selection.ListFormat.LeftIndent += textControl1.Selection.ListFormat.HangingIndent;
        }

        private void mnuFormat_BulletsAndNumbering_DecreaseLevel_Click(object sender, System.EventArgs e)
        {
            if (textControl1.Selection.ListFormat.Level >= 2)
            {
                textControl1.Selection.ListFormat.Level -= 1;
                textControl1.Selection.ListFormat.LeftIndent -= textControl1.Selection.ListFormat.HangingIndent;
            }
        }

        private void mnuFormat_BulletsAndNumbering_ArabicNumbers_Click(object sender, System.EventArgs e)
        {
            textControl1.Selection.ListFormat.Type = TXTextControl.ListType.Numbered;
            textControl1.Selection.ListFormat.NumberFormat = TXTextControl.NumberFormat.ArabicNumbers;
        }

        private void mnuFormat_BulletsAndNumbering_CapitalLetters_Click(object sender, System.EventArgs e)
        {
            textControl1.Selection.ListFormat.Type = TXTextControl.ListType.Numbered;
            textControl1.Selection.ListFormat.NumberFormat = TXTextControl.NumberFormat.CapitalLetters;
        }

        private void mnuFormat_BulletsAndNumbering_Letters_Click(object sender, System.EventArgs e)
        {
            textControl1.Selection.ListFormat.Type = TXTextControl.ListType.Numbered;
            textControl1.Selection.ListFormat.NumberFormat = TXTextControl.NumberFormat.Letters;
        }

        private void mnuFormat_BulletsAndNumbering_RomanNumbers_Click(object sender, System.EventArgs e)
        {
            textControl1.Selection.ListFormat.Type = TXTextControl.ListType.Numbered;
            textControl1.Selection.ListFormat.NumberFormat = TXTextControl.NumberFormat.RomanNumbers;
        }

        private void mnuFormat_BulletsAndNumbering_SmallRomanNumbers_Click(object sender, System.EventArgs e)
        {
            textControl1.Selection.ListFormat.Type = TXTextControl.ListType.Numbered;
            textControl1.Selection.ListFormat.NumberFormat = TXTextControl.NumberFormat.SmallRomanNumbers;
        }

        private void mnuFormat_BulletsAndNumbering_Bullets_Click(object sender, System.EventArgs e)
        {
            textControl1.Selection.ListFormat.Type = TXTextControl.ListType.Bulleted;
        }

        #endregion

        #region " Insert menu event handlers "

        private void mnuInsert_Popup(object sender, System.EventArgs e)
        {
            // Try/Catch is required because hypertext links are not available in the Standard version. 
            // of Text Control. Accessing them would throw an exception if this sample program is used 
            // with a Standard Version.
            try
            {
                mnuInsert_Hyperlink.Enabled = textControl1.HypertextLinks.CanAdd;
                mnuInsert_Target.Enabled = textControl1.DocumentTargets.CanAdd;
            }
            catch { }
        }

        private void mnuInsert_File_Click(object sender, System.EventArgs e)
        {
            try
            {
                FileHandler1.Insert();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "TX Text Control Words");
            }
        }

        private void mnuInsert_Image_Click(object sender, System.EventArgs e)
        {
            TXTextControl.Image NewImage = new TXTextControl.Image();
            textControl1.Images.Add(NewImage, TXTextControl.HorizontalAlignment.Left, -1, TXTextControl.ImageInsertionMode.DisplaceText);
        }

        private void mnuInsert_TextFrame_Click(object sender, System.EventArgs e)
        {
            Size TextFrameSize = new Size(2000, 1000);

            TXTextControl.TextFrame NewTextFrame = new TXTextControl.TextFrame(TextFrameSize);
            textControl1.TextFrames.Add(NewTextFrame, TXTextControl.HorizontalAlignment.Left, -1, TXTextControl.TextFrameInsertionMode.DisplaceText);

			//RapidSpell-
            //Must inform helper of this
            helper.OnTextFrameCreated(NewTextFrame);
			//-
        }

        private void mnuInsert_Hyperlink_Click(object sender, System.EventArgs e)
        {
            try
            {
                frmHyperlink HyperlinksDialog = new frmHyperlink();

                HyperlinksDialog.txMain = textControl1;
                HyperlinksDialog.ShowDialog();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "TX Text Control Words");
            }
        }

        private void mnuInsert_Target_Click(object sender, System.EventArgs e)
        {
            try
            {
                string TargetName = "";

                if (InputBox.ShowInputBox("Target Name:", ref TargetName))
                {
                    TXTextControl.DocumentTarget Target = new TXTextControl.DocumentTarget(TargetName);
                    textControl1.DocumentTargets.Add(Target);
                    FileHandler1.DocumentDirty = true;
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "TX Text Control Words");
            }
        }

        #endregion

        #region " View menu event handlers "

        private void mnuView_Popup(object sender, System.EventArgs e)
        {
            mnuView_Draft.Checked = (textControl1.ViewMode == TXTextControl.ViewMode.Normal);
            mnuView_PageLayout.Checked = (textControl1.ViewMode == TXTextControl.ViewMode.PageView);
            mnuView_Toolbar.Checked = toolStrip1.Visible;
            mnuView_ButtonBar.Checked = buttonBar1.Visible;
            mnuView_StatusBar.Checked = statusBar1.Visible;
            mnuView_HorizontalRuler.Checked = rulerBar1.Visible;
            mnuView_VerticalRuler.Checked = rulerBar2.Visible;
            mnuView_HeadersAndFooters.Enabled = (textControl1.ViewMode == TXTextControl.ViewMode.PageView);
        }

        private void mnuView_Normal_Click(object sender, System.EventArgs e)
        {
            textControl1.ViewMode = TXTextControl.ViewMode.Normal;
        }

        private void mnuView_PageLayout_Click(object sender, System.EventArgs e)
        {
            textControl1.ViewMode = TXTextControl.ViewMode.PageView;
        }

        private void mnuView_HeadersAndFooters_Click(object sender, System.EventArgs e)
        {
            try
            {
                TXTextControl.HeaderFooter Header;
                if (textControl1.HeadersAndFooters.Count == 0)
                {
                    textControl1.HeadersAndFooters.Add(TXTextControl.HeaderFooterType.Header);
                }
                textControl1.HeaderFooterActivationStyle = TXTextControl.HeaderFooterActivationStyle.ActivateClick;
                Header = textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Header);
                
                if ((Header != null))
                {
                    Header.Activate();
                    
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "TX Text Control Words");
            }
        }

        private void mnuView_Toolbar_Click(object sender, System.EventArgs e)
        {
            toolStrip1.Visible = !toolStrip1.Visible;
        }

        private void mnuView_ButtonBar_Click(object sender, System.EventArgs e)
        {
            buttonBar1.Visible = !buttonBar1.Visible;
        }

        private void mnuView_StatusBar_Click(object sender, System.EventArgs e)
        {
            statusBar1.Visible = !statusBar1.Visible;
        }

        private void mnuView_HorizontalRuler_Click(object sender, System.EventArgs e)
        {
            rulerBar1.Visible = !rulerBar1.Visible;
        }

        private void mnuView_VerticalRuler_Click(object sender, System.EventArgs e)
        {
            rulerBar2.Visible = !rulerBar2.Visible;
        }

        private void mnuView_Zoom_Click(object sender, System.EventArgs e)
        {
            mnuView_Zoom_25.Checked = (textControl1.ZoomFactor == 25);
            mnuView_Zoom_50.Checked = (textControl1.ZoomFactor == 50);
            mnuView_Zoom_75.Checked = (textControl1.ZoomFactor == 75);
            mnuView_Zoom_100.Checked = (textControl1.ZoomFactor == 100);
            mnuView_Zoom_150.Checked = (textControl1.ZoomFactor == 150);
            mnuView_Zoom_200.Checked = (textControl1.ZoomFactor == 200);
            mnuView_Zoom_300.Checked = (textControl1.ZoomFactor == 300);
        }

        private void mnuView_Zoom_25_Click(object sender, System.EventArgs e)
        {
            textControl1.ZoomFactor = 25;
        }

        private void mnuView_Zoom_50_Click(object sender, System.EventArgs e)
        {
            textControl1.ZoomFactor = 50;
        }

        private void mnuView_Zoom_75_Click(object sender, System.EventArgs e)
        {
            textControl1.ZoomFactor = 75;
        }

        private void mnuView_Zoom_100_Click(object sender, System.EventArgs e)
        {
            textControl1.ZoomFactor = 100;
        }

        private void mnuView_Zoom_150_Click(object sender, System.EventArgs e)
        {
            textControl1.ZoomFactor = 150;
        }

        private void mnuView_Zoom_200_Click(object sender, System.EventArgs e)
        {
            textControl1.ZoomFactor = 200;
        }

        private void mnuView_Zoom_300_Click(object sender, System.EventArgs e)
        {
            textControl1.ZoomFactor = 300;
        }

        #endregion

        #region " Edit menu event handlers "

        private void mnuEdit_Popup(object sender, System.EventArgs e)
        {
            mnuEdit_Undo.Enabled = textControl1.CanUndo;
            mnuEdit_Redo.Enabled = textControl1.CanRedo;
            mnuEdit_Cut.Enabled = textControl1.CanCopy;
            mnuEdit_Copy.Enabled = textControl1.CanCopy;
            mnuEdit_Paste.Enabled = textControl1.CanPaste;

            mnuEdit_Undo.Text = "Undo " + textControl1.UndoActionName;
            mnuEdit_Redo.Text = "Redo " + textControl1.RedoActionName;

            // Hypertext links are not available in the Standard version. Accessing them  
            // would throw an exception if this sample program is used with a Standard version
            // of Text Control.
            try
            {
                mnuEdit_Hyperlink.Enabled = (textControl1.HypertextLinks.GetItem() != null) || (textControl1.DocumentLinks.GetItem() != null);
                mnuEdit_Target.Enabled = (textControl1.DocumentTargets.GetItem() != null);
            }
            catch
            { }
        }

        private void mnuEdit_Undo_Click(object sender, System.EventArgs e)
        {
            Undo();
        }

        private void mnuEdit_Redo_Click(object sender, System.EventArgs e)
        {
            Redo();
        }

        private void mnuEdit_Cut_Click(object sender, System.EventArgs e)
        {
            Cut();
        }

        private void mnuEdit_Copy_Click(object sender, System.EventArgs e)
        {
            Copy();
        }

        private void mnuEdit_Paste_Click(object sender, System.EventArgs e)
        {
            Paste();
        }

        private void mnuEdit_Delete_Click(object sender, System.EventArgs e)
        {
            textControl1.Clear();
        }

        private void mnuEdit_SelectAll_Click(object sender, System.EventArgs e)
        {
            textControl1.SelectAll();
        }

        private void mnuEdit_Find_Click(object sender, System.EventArgs e)
        {
            Find();
        }

        private void mnuEdit_Replace_Click(object sender, System.EventArgs e)
        {
            textControl1.Replace();
        }

        private void mnuEdit_Hyperlink_Click(object sender, System.EventArgs e)
        {
            try
            {
                frmHyperlink HyperlinksDialog = new frmHyperlink();

                HyperlinksDialog.txMain = textControl1;
                HyperlinksDialog.ShowDialog();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "TX Text Control Words");
            }
        }

        private void mnuEdit_Target_Click(object sender, System.EventArgs e)
        {
            try
            {
                string TargetName = "";
                TXTextControl.DocumentTarget Target = textControl1.DocumentTargets.GetItem();

                if (Target != null)
                {
                    TargetName = Target.Name;
                    if (InputBox.ShowInputBox("Enter a name for this target:", ref TargetName))
                    {
                        Target.Name = TargetName;
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "TX Text Control Words");
            }
        }
        #endregion

        #region " Form Events "

        private void frmMain_Load(object sender, EventArgs e)
        {
            LoadAppSettings();
            
            FileHandler1.TextControl = textControl1;
            
            FileHandler1.RecentFilesMenuItem = recentFilesToolStripMenuItem;

            //RapidSpell-
            //create header/footer now
            TXTextControl.HeaderFooterCollection hfc = this.textControl1.HeadersAndFooters;
            textControl1.HeaderFooterActivationStyle = TXTextControl.HeaderFooterActivationStyle.ActivateClick;
            textControl1.HeaderFooterFrameStyle = TXTextControl.HeaderFooterFrameStyle.DottedFrame;
            hfc.Add(TXTextControl.HeaderFooterType.Header);
              hfc.Add(TXTextControl.HeaderFooterType.Footer);

            //the second argument should be RapidSpellDialog if it is being used aswell           
            helper = new TXHelper(rapidSpellAsYouType1, rapidSpellDialog1, this.textControl1, ContextMenu_Text);
            
            //-

            if (m_LoadFileOnCreate)
            {
                if (!FileHandler1.FileOpen())
                    return;
                if (FileHandler1.DocumentFileName != "")
                    this.Text = FileHandler1.DocumentFileName;
            }

            FileHandler1.DocumentDirty = false;
            UpdateSaveStatus();
        }

        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            System.Windows.Forms.DialogResult Result;

            if (FileHandler1.DocumentDirty)
            {
                Result = MessageBox.Show("Save changes to " + this.Text + "?", "Question", MessageBoxButtons.YesNoCancel);
                if (Result == DialogResult.Yes)
                {
                    FileHandler1.FileSave();
                    if (FileHandler1.DocumentFileName == "")
                        e.Cancel = true;
                }
                else if (Result == DialogResult.Cancel)
                    e.Cancel = true;
            }
            SaveAppSettings();
        }
        #endregion

        #region " TextControl events "

        private void textControl1_Changed(object sender, System.EventArgs e)
        {
            FileHandler1.DocumentDirty = true;
            EnableToolbarButtons();
            UpdateSaveStatus();
        }

        private void textControl1_DocumentLinkClicked(object sender, TXTextControl.DocumentLinkEventArgs e)
        {
            e.DocumentLink.ScrollTo();
        }

        private void textControl1_HypertextLinkClicked(object sender, TXTextControl.HypertextLinkEventArgs e)
        {
            if (e.HypertextLink.Target.StartsWith("http"))
            {
                System.Diagnostics.Process.Start(e.HypertextLink.Target);
            }
        }

        private void textControl1_InputPositionChanged(object sender, System.EventArgs e)
        {
            EnableToolbarButtons();
        }

        private void textControl1_HeaderFooterActivated(object sender, TXTextControl.HeaderFooterEventArgs e)
        {
            m_ActiveHeaderFooter = e.HeaderFooter;
            EnableToolbarButtons();
        }

        private void textControl1_HeaderFooterDeactivated(object sender, TXTextControl.HeaderFooterEventArgs e)
        {
            m_ActiveHeaderFooter = null;
            EnableToolbarButtons();
        }

        private void textControl1_GotFocus(object sender, System.EventArgs e)
        {
            EnableToolbarButtons();
        }

        #endregion

        #region "  Menu item and toolbar button functions  "

        public void FileSave()
        {
            try
            {
                FileHandler1.FileSave();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "TX Text Control Words");
            }
            UpdateSaveStatus();
        }

        public void FileNew()
        {
            System.Windows.Forms.DialogResult Result;

            if (FileHandler1.DocumentDirty)
            {
                Result = MessageBox.Show("Save changes to " + this.Text + "?", "Question", MessageBoxButtons.YesNoCancel);
                if (Result == DialogResult.Yes)
                {
                    FileHandler1.FileSave();
                    if (FileHandler1.DocumentFileName == "")
                        return;
                }
                else if (Result == DialogResult.Cancel)
                    return;
            }

            FileHandler1.DocumentFileName = "";
            textControl1.ResetContents();
            this.Text = "TX Text Control Words - [untitled]";
            FileHandler1.DocumentDirty = false;
            UpdateSaveStatus();
        }

        public void FileOpen()
        {
            System.Windows.Forms.DialogResult Result;

            if (FileHandler1.DocumentDirty)
            {
                Result = MessageBox.Show("Save changes to " + this.Text + "?", "Question", MessageBoxButtons.YesNoCancel);
                if (Result == DialogResult.Yes)
                {
                    FileHandler1.FileSave();
                    if (FileHandler1.DocumentFileName == "")
                        return;
                }
                else if (Result == DialogResult.Cancel)
                    return;
            }

            FileHandler1.DocumentFileName = "";
            FileHandler1.FileOpen();
            UpdateSaveStatus();

            //RapidSpell-
            helper.OnDocumentOpen();
            //-
        }

        public void Print()
        {
            textControl1.Print("TX Text Control Words Document");
        }

        public void PrintPreview()
        {
            textControl1.PrintPreview("TX Text Control Words Document");
        }

        public void Cut()
        {
            textControl1.Cut();
        }

        public void Copy()
        {
            textControl1.Copy();
        }

        public void Paste()
        {
            textControl1.Paste();
        }

        public void Undo()
        {
            textControl1.Undo();
        }

        public void Redo()
        {
            textControl1.Redo();
        }

        public void Find()
        {
            textControl1.Find();
        }

        public void About()
        {
            AssemblyCopyrightAttribute applicationCopyright =(AssemblyCopyrightAttribute)AssemblyCopyrightAttribute.GetCustomAttribute(System.Reflection.Assembly.GetExecutingAssembly(), typeof(AssemblyCopyrightAttribute));
            MessageBox.Show(this.ProductName.ToString()
            + "\r\n"
            + applicationCopyright.Copyright.ToString()
            + "\r\n"
            + "\r\n"
            + "Icons by Mark James, www.famfamfam.com", "About TX Text Control Words", MessageBoxButtons.OK); 
        }

        public void RemoveHeaderFooter()
        {
            if ((m_ActiveHeaderFooter != null))
            {
                textControl1.HeadersAndFooters.Remove(m_ActiveHeaderFooter.Type);
            }
        }

        public void SwitchBetweenHeaderAndFooter()
        {
            if ((m_ActiveHeaderFooter != null))
            {
                try
                {
                    switch (m_ActiveHeaderFooter.Type)
                    {
                        case TXTextControl.HeaderFooterType.Footer:
                            if ((textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Header) != null))
                            {
                                textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Header).Activate();
                            }
                            else
                            {
                                textControl1.HeadersAndFooters.Add(TXTextControl.HeaderFooterType.Header);
                                textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Header).Activate();
                            }

                            break;

                        case TXTextControl.HeaderFooterType.Header:
                            if ((textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Footer) != null))
                            {
                                textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Footer).Activate();
                            }
                            else
                            {
                                textControl1.HeadersAndFooters.Add(TXTextControl.HeaderFooterType.Footer);
                                textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Footer).Activate();
                            }

                            break;

                        case TXTextControl.HeaderFooterType.FirstPageFooter:
                            if ((textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageHeader) != null))
                            {
                                textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageHeader).Activate();
                            }
                            else
                            {
                                textControl1.HeadersAndFooters.Add(TXTextControl.HeaderFooterType.FirstPageHeader);
                                textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageHeader).Activate();
                            }

                            break;

                        case TXTextControl.HeaderFooterType.FirstPageHeader:
                            if ((textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageFooter) != null))
                            {
                                textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageFooter).Activate();
                            }
                            else
                            {
                                textControl1.HeadersAndFooters.Add(TXTextControl.HeaderFooterType.FirstPageFooter);
                                textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageFooter).Activate();
                            }

                            break;
                    }
                }
                finally { }
            }
        }

        public void GoToFirstPage()
        {
            if ((m_ActiveHeaderFooter != null))
            {
                try
                {
                    switch (m_ActiveHeaderFooter.Type)
                    {
                        case TXTextControl.HeaderFooterType.Header:
                            if ((textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageHeader) != null))
                            {
                                textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageHeader).Activate();
                            }

                            break;
                        case TXTextControl.HeaderFooterType.Footer:
                            if ((textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageFooter) != null))
                            {
                                textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageFooter).Activate();
                            }

                            break;
                    }
                }
                finally { }
            }
        }

        public void GoToDefault()
        {
            if ((m_ActiveHeaderFooter != null))
            {
                try
                {
                    switch (m_ActiveHeaderFooter.Type)
                    {
                        case TXTextControl.HeaderFooterType.FirstPageHeader:
                            if ((textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Header) != null))
                            {
                                textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Header).Activate();
                            }
                            else
                            {
                                textControl1.HeadersAndFooters.Add(TXTextControl.HeaderFooterType.Header);
                                textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Header).Activate();
                            }

                            break;
                        case TXTextControl.HeaderFooterType.FirstPageFooter:
                            if ((textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Footer) != null))
                            {
                                textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Footer).Activate();
                            }
                            else
                            {
                                textControl1.HeadersAndFooters.Add(TXTextControl.HeaderFooterType.Footer);
                                textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Footer).Activate();
                            }

                            break;
                    }
                }
                finally { }
            }
        }

        public int GetNumberOfPages()
        {
            int pages = 0;
            for (int i = 1; i <= textControl1.Pages; i++)
            {
                if(textControl1.GetPages()[i].Section == textControl1.Sections.GetItem().Number)
                {
                    pages += 1;
                }
            }
            return pages;
        }


        #endregion

        # region "  Form functions  "
        private void LoadAppSettings()
        {
            // Take over initial resizing
            this.StartPosition = FormStartPosition.Manual;

            // Resize form
            this.Size = Properties.Settings.Default.LastWindowSize;
            this.Location = Properties.Settings.Default.LastWindowPos;
            this.WindowState = Properties.Settings.Default.LastWindowState;
            
        }

        private void SaveAppSettings()
        {
            // Store location and size data, using RestoreBounds to remember normal position if minimized or maximized
            
            if (this.WindowState == FormWindowState.Normal)
            {
                Properties.Settings.Default.LastWindowPos = this.Location;
                Properties.Settings.Default.LastWindowSize = this.Size;
            }
            else
            {
                Properties.Settings.Default.LastWindowPos =
                  this.RestoreBounds.Location;
                Properties.Settings.Default.LastWindowSize =
                  this.RestoreBounds.Size;
            }
            Properties.Settings.Default.LastWindowState = this.WindowState;
            Properties.Settings.Default.Save();
        }

        #endregion

        #region " Toolbar event handlers "

        public void UpdateSaveStatus()
        {
            toolStrip1.Items[2].Enabled = FileHandler1.DocumentDirty;
            mnuFile.DropDownItems[4].Enabled = FileHandler1.DocumentDirty;
        }

        public void EnableToolbarButtons()
        {
            toolStrip1.Items[7].Enabled = this.CanCopy;
            toolStrip1.Items[8].Enabled = this.CanCopy;
            toolStrip1.Items[9].Enabled = this.CanPaste;
            toolStrip1.Items[10].Enabled = this.CanCopy;
            toolStrip1.Items[12].Enabled = this.CanUndo;
            toolStrip1.Items[13].Enabled = this.CanRedo;
            if ((m_ActiveHeaderFooter != null))
            {
                toolStrip1.Items[16].Enabled = true;
                toolStrip1.Items[17].Enabled = !((textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.Header) == null)) & (GetNumberOfPages() >= 2) & m_ActiveHeaderFooter.Type != TXTextControl.HeaderFooterType.Header;
                toolStrip1.Items[18].Enabled = !((textControl1.HeadersAndFooters.GetItem(TXTextControl.HeaderFooterType.FirstPageHeader) == null)) & m_ActiveHeaderFooter.Type != TXTextControl.HeaderFooterType.FirstPageHeader;
                toolStrip1.Items[19].Enabled = true;
            }
            else
            {
                toolStrip1.Items[16].Enabled = false;
                toolStrip1.Items[17].Enabled = false;
                toolStrip1.Items[18].Enabled = false;
                toolStrip1.Items[19].Enabled = false;
            }
        }


        private void toolStripButton1_Click(object sender, EventArgs e)
        {
            FileNew();
        }

        private void toolStripButton2_Click(object sender, EventArgs e)
        {
            FileOpen();
        }

        private void toolStripButton3_Click(object sender, EventArgs e)
        {
            FileSave();
        }

        private void toolStripButton4_Click(object sender, EventArgs e)
        {
            Print();
        }

        private void toolStripButton5_Click(object sender, EventArgs e)
        {
            PrintPreview();
        }

        private void toolStripButton6_Click(object sender, EventArgs e)
        {
            Cut();
        }

        private void toolStripButton7_Click(object sender, EventArgs e)
        {
            Copy();
        }

        private void toolStripButton8_Click(object sender, EventArgs e)
        {
            Paste();
        }

        private void toolStripButton9_Click(object sender, EventArgs e)
        {
            textControl1.Clear();
        }

        private void toolStripButton10_Click(object sender, EventArgs e)
        {
            Undo();
        }

        private void toolStripButton11_Click(object sender, EventArgs e)
        {
            Redo();
        }

        private void toolStripButton12_Click(object sender, EventArgs e)
        {
            Find();
        }

        private void toolStripButton13_Click(object sender, EventArgs e)
        {
            SwitchBetweenHeaderAndFooter();
        }

        private void toolStripButton15_Click(object sender, EventArgs e)
        {
            GoToDefault();
        }

        private void toolStripButton16_Click(object sender, EventArgs e)
        {
            GoToFirstPage();
        }

        private void toolStripButton14_Click(object sender, EventArgs e)
        {
            RemoveHeaderFooter();
        }

        #endregion

        #region " Context menus "

        private void ContextMenu_Text_Popup(object sender, System.EventArgs e)
        {
            ContextMenu_Text_TableProperties.Enabled = (textControl1.Tables.GetItem() != null);
            ContextMenu_Text_Cut.Enabled = textControl1.CanCopy;
            ContextMenu_Text_Copy.Enabled = textControl1.CanCopy;
            ContextMenu_Text_Paste.Enabled = textControl1.CanPaste;

            ToolStripItem[] suggs = thesaurus1.GetSuggestionToolStripItems();
            synonymsToolStripMenuItem.DropDown.Items.Clear();
            //If we have suggestions, add them too
            if (suggs != null)
            {
                synonymsToolStripMenuItem.DropDown.Items.AddRange(suggs);
            }
            else
            {
                ToolStripMenuItem noSuggs = new ToolStripMenuItem("No suggestions");
                noSuggs.Enabled = false;
                synonymsToolStripMenuItem.DropDown.Items.Add(noSuggs);
            }
        }

        private void ContextMenu_Images_Popup(object sender, System.EventArgs e)
        {
            ContextMenu_Images_Cut.Enabled = textControl1.CanCopy;
            ContextMenu_Images_Copy.Enabled = textControl1.CanCopy;
            ContextMenu_Images_Paste.Enabled = textControl1.CanPaste;
        }

        private void ContextMenu_TextFrames_Popup(object sender, System.EventArgs e)
        {
            ContextMenu_TextFrames_Cut.Enabled = textControl1.CanCopy;
            ContextMenu_TextFrames_Copy.Enabled = textControl1.CanCopy;
            ContextMenu_TextFrames_Paste.Enabled = textControl1.CanPaste;
        }

        private void textControl1_ImageRightClicked(object sender, TXTextControl.ImageEventArgs e)
        {
            ContextMenu_Images.Show(this, this.PointToClient(Cursor.Position));
        }

        private void textControl1_TextFrameRightClicked(object sender, TXTextControl.TextFrameEventArgs e)
        {/*RapidSpell-
            if (textControl1.TextFrames.GetItem() == null)

                ContextMenu_Text.Show(this, this.PointToClient(Cursor.Position));

            else
                ContextMenu_TextFrames.Show(this, this.PointToClient(Cursor.Position));
 */
        }

        private void textControl1_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            /*RapidSpell-
            if (e.Button == MouseButtons.Right)
                txTextControlAdapter1.ContextMenuStripDefault = ContextMenu_Text;

            */
        }

        private void frmMDIChild_Activated(object sender, System.EventArgs e)
        {
            if (textControl1.CanFocus)
            {
                textControl1.Focus();
            }
        }
        #endregion

        #region " Menus "

        private void ContextMenu_TextFrames_Cut_Click(object sender, EventArgs e)
        {
            Cut();
        }

        private void ContextMenu_TextFrames_Copy_Click(object sender, EventArgs e)
        {
            Copy();
        }

        private void ContextMenu_TextFrames_Paste_Click(object sender, EventArgs e)
        {
            Paste();
        }

        private void ContextMenu_TextFrames_FormatTextFrame_Click(object sender, EventArgs e)
        {
            textControl1.TextFrameAttributesDialog();
        }

        private void ContextMenu_Images_Cut_Click(object sender, EventArgs e)
        {
            Cut();
        }

        private void ContextMenu_Images_Copy_Click(object sender, EventArgs e)
        {
            Copy();
        }

        private void ContextMenu_Images_Paste_Click(object sender, EventArgs e)
        {
            Paste();
        }

        private void ContextMenu_Images_FormatImage_Click(object sender, EventArgs e)
        {
            textControl1.ImageAttributesDialog();
        }

        private void ContextMenu_Text_Cut_Click(object sender, EventArgs e)
        {
            Cut();
        }

        private void ContextMenu_Text_Copy_Click(object sender, EventArgs e)
        {
            Copy();
        }

        private void ContextMenu_Text_Paste_Click(object sender, EventArgs e)
        {
            Paste();
        }

        private void ContextMenu_Text_Character_Click(object sender, EventArgs e)
        {
            textControl1.FontDialog();
        }

        private void ContextMenu_Text_Paragraph_Click(object sender, EventArgs e)
        {
            textControl1.ParagraphFormatDialog();
        }

        private void ContextMenu_Text_BulletsAndNumbering_Click(object sender, EventArgs e)
        {
            textControl1.ListFormatDialog();
        }

        private void ContextMenu_Text_InsertTable_Click(object sender, EventArgs e)
        {
            frmInsertTable InsertTableDialog = new frmInsertTable();

            InsertTableDialog.tx = textControl1;
            InsertTableDialog.ShowDialog();
        }

        private void ContextMenu_Text_TableProperties_Click(object sender, EventArgs e)
        {
            textControl1.TableFormatDialog();
        }

        private void sectionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            frmInsertBreak frmInsertBreak = new frmInsertBreak();

            frmInsertBreak.tx = textControl1;
            frmInsertBreak.ShowDialog();
        }

        #endregion

        

        private void sToolStripMenuItem_Click(object sender, EventArgs e)
        {
        	//RapidSpell-
            rapidSpellDialog1.Check();
            //-
        }

        private void thesaurusToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (this.textControl1.Text.Length == 0)
                MessageBox.Show("You must select a word to use the thesaurus.");
            else
            {
                this.thesaurus1.DialogForm.TopMost = true;
                this.thesaurus1.Show();
            }
        }

  
    }
}
